@@ -1,17 +1,16 @@
 #!/usr/bin/env python3
-"""An open library for simplified-traditional Chinese text conversion.
+"""An open library for flexible simplified-traditional Chinese text conversion.
 """
 import sys
 import os
 import re
-import glob
 import argparse
 import json
 import html
 from collections import namedtuple, OrderedDict
 import math
 
-__version__ = '0.16.0'
+__version__ = '0.18.0'
 __author__ = 'Danny Lin'
 __author_email__ = 'danny0838@gmail.com'
 __homepage__ = 'https://github.com/danny0838/sts-lib'
@@ -139,7 +138,7 @@ class Unicode():
 
     @staticmethod
     def split(text):
-        """Split text into a list of Unicode composites.
+        """Split a text into a list of Unicode composites.
         """
         i = 0
         total = len(text)
@@ -150,7 +149,7 @@ class Unicode():
             i += length
         return result
 
-class StsDict(OrderedDict):
+class StsDict():
     """Base class of an STS dictionary.
 
     This class is for child classes to implement on and not intended to be
@@ -164,22 +163,64 @@ class StsDict(OrderedDict):
           ...
 
     - JSON: which is dumped from the internal data structure.
-
-    NOTE: The internal data structure may vary across subclasses. As a result,
-    loadjson(), dumpjson(), and most native dict methods like __init__(),
-    __iter__(), items(), keys(), and values() work on their own format.
-
-    To safely create a subclass from a dict-compatible data format, do
-    something like:
-
-        SubClass().add_dict(StsDict(...))
-
-    Or use load() or load_filegroups() to load from a plain-dict file.
     """
-    def iter(self):
+    def __init__(self, *args, **kwargs):
+        self._dict = {}
+        for key, values in dict(*args, **kwargs).items():
+            self.add(key, values, skip_check=True)
+
+    def __repr__(self):
+        """Implementation of repr(self).
+        """
+        return f"{self.__class__.__name__}({repr(list(self.items()))})"
+
+    def __getitem__(self, key):
+        """Implementation of self[key].
+        """
+        return self._dict[key]
+
+    def __contains__(self, item):
+        """Implementation of "item in self".
+        """
+        return item in self._dict
+
+    def __len__(self):
+        """Implementation of len(self).
+        """
+        return len(self._dict)
+
+    def __iter__(self):
+        """Implementation of iter(self).
+        """
+        yield from self._dict
+
+    def __eq__(self, other):
+        """Implementation of "==" operator.
+        """
+        if not isinstance(other, (dict, StsDict)):
+            return False
+        for key, value in self.items():
+            try:
+                if not value == other[key]:
+                    return False
+            except KeyError:
+                return False
+        return True
+
+    def keys(self):
+        """Get a generator of keys.
+        """
+        yield from self._dict.keys()
+
+    def values(self):
+        """Get a generator of values.
+        """
+        yield from self._dict.values()
+
+    def items(self):
         """Get a generator of key-values pairs.
         """
-        yield from self.items()
+        yield from self._dict.items()
 
     def add(self, key, values, skip_check=False):
         """Add a key-values pair to this dictionary.
@@ -189,20 +230,17 @@ class StsDict(OrderedDict):
             skip_check: True to skip checking duplicated values.
         """
         values = [values] if isinstance(values, str) else values
-        list_ = self.setdefault(key, [])
-        if skip_check:
-            list_ += values
-        else:
-            list_ += [x for x in values if x not in list_]
+        list_ = self._dict.setdefault(key, [])
+        list_ += values if skip_check else [x for x in values if x not in list_]
         return self
 
-    def add_dict(self, stsdict, skip_check=False):
-        """Add all key-values pairs from another StsDict.
+    def update(self, stsdict, skip_check=False):
+        """Add all key-values pairs from another StsDict or dict.
 
         Args:
             skip_check: True to skip checking duplicated values.
         """
-        for key, values in stsdict.iter():
+        for key, values in stsdict.items():
             self.add(key, values, skip_check=skip_check)
         return self
 
@@ -212,9 +250,11 @@ class StsDict(OrderedDict):
         for file in files:
             with open(file, "r", encoding="UTF-8") as f:
                 for line in f:
-                    parts = line.strip().split("\t")
-                    if len(parts) > 1:
-                        key, values = parts[0], parts[1]
+                    try:
+                        key, values, *_ = line.strip().split("\t")
+                    except ValueError:
+                        pass
+                    else:
                         self.add(key, values.split(" "))
                 f.close()
         return self
@@ -227,7 +267,7 @@ class StsDict(OrderedDict):
             sort: True to sort the output.
         """
         f = open(file, "w", encoding="UTF-8", newline="") if file else sys.stdout
-        iterator = self.iter()
+        iterator = self.items()
         if sort: iterator = sorted(iterator)
         for key, values in iterator:
             f.write(f'{key}\t{" ".join(values)}\n')
@@ -242,7 +282,8 @@ class StsDict(OrderedDict):
             a new object with the same class.
         """
         with open(file, 'r', encoding='UTF-8') as f:
-            stsdict = self.__class__(json.load(f))
+            stsdict = self.__class__()
+            stsdict._dict = json.load(f)
             f.close()
         return stsdict
 
@@ -257,7 +298,7 @@ class StsDict(OrderedDict):
             sort: True to sort the output.
         """
         f = open(file, "w", encoding="UTF-8", newline="") if file else sys.stdout
-        json.dump(self, f, ensure_ascii=False, indent=indent, sort_keys=sort)
+        json.dump(self._dict, f, ensure_ascii=False, indent=indent, sort_keys=sort)
         if f is not sys.stdout: f.close()
 
     def print(self, sort=False):
@@ -266,7 +307,7 @@ class StsDict(OrderedDict):
         Args:
             sort: True to sort the output.
         """
-        iterator = self.iter()
+        iterator = self.items()
         if sort: iterator = sorted(iterator)
         for key, values in iterator:
             print(f'{key} => {" ".join(values)}')
@@ -282,7 +323,7 @@ class StsDict(OrderedDict):
         Returns:
             a generator of matched key-values pairs.
         """
-        for key, values in self.iter():
+        for key, values in self.items():
             if exact:
                 if from_keys and keyword == key:
                     yield key, values
@@ -303,7 +344,7 @@ class StsDict(OrderedDict):
             a new object with the same class.
         """
         stsdict = self.__class__()
-        for key, values in self.iter():
+        for key, values in self.items():
             for value in values:
                 stsdict.add(value, key)
         return stsdict
@@ -334,12 +375,15 @@ class StsDict(OrderedDict):
         Applying the returned dictionary to a text simulates the effect
         of applying using self and then using stsdict.
 
+        This will invoke stsdict.apply_enum and it's recommended that stsdict
+        be Table or Trie class for better performance.
+
         Returns:
             a new object with the same class.
         """
         dict1 = self._join_postfix(stsdict)
         dict2 = stsdict._join_prefix(self)
-        return dict1.add_dict(dict2)
+        return dict1.update(dict2)
 
     def _join_prefix(self, stsdict):
         """Prefix self with stsdict.
@@ -364,7 +408,7 @@ class StsDict(OrderedDict):
         """
         dict_ = self.__class__()
         converter = stsdict.swap()
-        for key, values in self.iter():
+        for key, values in self.items():
             for newkey in converter.apply_enum(key, include_short=True, include_self=True):
                 dict_.add(newkey, values)
         return dict_
@@ -388,12 +432,24 @@ class StsDict(OrderedDict):
             爲 => 為
         """
         dict_ = self.__class__()
-        for key, values in self.iter():
+        for key, values in self.items():
             for value in values:
                 dict_.add(key, stsdict.apply_enum(value))
-        dict_.add_dict(stsdict)
+        dict_.update(stsdict)
         return dict_
 
+    def _split(self, parts):
+        """Split parts into a list of Unicode composites.
+
+        With automatic type handling.
+        """
+        if isinstance(parts, str):
+            return Unicode.split(parts)
+        elif isinstance(parts, list):
+            return parts
+        else:
+            return list(parts)
+
     def match(self, parts, pos, maxlen=math.inf):
         """Match a unicode composite at pos.
 
@@ -403,16 +459,18 @@ class StsDict(OrderedDict):
         Returns:
             an StsDictMatch or None if no match.
         """
-        parts = (Unicode.split(parts) if isinstance(parts, str) else
-                parts if isinstance(parts, list) else
-                list(parts))
-        i = max(len(Unicode.split(key)) for key in self)
+        parts = self._split(parts)
+        i = max(len(Unicode.split(key)) for key in self._dict)
         i = min(i, min(len(parts), maxlen) - pos)
         while i >= 1:
             end = pos + i
-            current = "".join(parts[pos:end])
-            if current in self:
-                conv = StsDictConv(current, self[current])
+            current_parts = parts[pos:end]
+            current = "".join(current_parts)
+            try:
+                conv = StsDictConv(current_parts, self._dict[current])
+            except KeyError:
+                pass
+            else:
                 return StsDictMatch(conv, pos, end)
             i -= 1
         return None
@@ -426,9 +484,7 @@ class StsDict(OrderedDict):
         Returns:
              a generator of parts, which is a string or an StsDictConv.
         """
-        parts = (Unicode.split(parts) if isinstance(parts, str) else
-                parts if isinstance(parts, list) else
-                list(parts))
+        parts = self._split(parts)
         i = 0
         total = len(parts)
         while i < total:
@@ -453,50 +509,44 @@ class StsDict(OrderedDict):
 
         e.g.
         table:
-            钟 => 鍾 鐘
+            钟 => 鐘 鍾
             药 => 藥 葯
             用药 => 用藥
         text:
             '看钟用药'
         table.apply_enum(text, include_short=False, include_self=False):
-            ['看鍾用藥', '看鐘用藥']
+            ['看鐘用藥', '看鍾用藥']
         table.apply_enum(text, include_short=True, include_self=False):
-            ['看鍾用藥', '看鐘用藥', '看鍾用葯', '看鐘用葯']
+            ['看鐘用藥', '看鐘用葯', '看鍾用藥', '看鍾用葯']
         table.apply_enum(text, include_short=False, include_self=True):
-            ['看鍾用藥', '看鍾用药', '看鐘用藥', '看鐘用药', '看钟用藥', '看钟用药']
+            ['看鐘用藥', '看鐘用药', '看鍾用藥', '看鍾用药', '看钟用藥', '看钟用药']
         table.apply_enum(text, include_short=True, include_self=True):
-            ['看鍾用藥', '看鍾用药', '看鐘用藥', '看鐘用药', '看钟用藥', '看钟用药', '看鍾用葯', '看鐘用葯', '看钟用葯']
+            ['看鐘用藥', '看鐘用药', '看鐘用葯', '看鍾用藥', '看鍾用药', '看鍾用葯', '看钟用藥', '看钟用药', '看钟用葯']
         """
-        if isinstance(parts, str):
-            text = parts
-            parts = Unicode.split(parts)
-        elif isinstance(parts, list):
-            text = None
-        else:
-            text = None
-            parts = list(parts)
+        text = parts
+        parts = self._split(parts)
 
         queue = [(parts, 0, 0)]
+        subqueue = []
         results = OrderedDict()
         while len(queue):
-            (parts, matched, nextindex) = data = queue.pop(0)
+            (parts, matched, nextindex) = data = queue.pop()
             if nextindex < len(parts):
-                self._apply_enum_sub(queue, data, include_short=include_short, include_self=include_self)
+                self._apply_enum_sub(subqueue, data, include_short=include_short, include_self=include_self)
+                while len(subqueue):
+                    queue.append(subqueue.pop())
             elif matched > 0:
                 results["".join(parts)] = True
 
         results = list(results)
 
         if len(results) == 0:
-            results.append(text if text is not None else "".join(parts))
+            results.append(text if isinstance(text, str) else "".join(text))
 
         return results
 
     def _apply_enum_sub(self, queue, data, include_short=False, include_self=False):
         """Helper function of apply_enum
-
-        Args:
-            data: (parts, matched, index)
         """
         (parts, matched, index) = data
         match = self.match(parts, index)
@@ -509,8 +559,10 @@ class StsDict(OrderedDict):
         has_atomic_match = False
         if match.end - index == 1:
             has_atomic_match = True
-        if include_self and match.conv.key not in match.conv.values:
-            match.conv.values.append(match.conv.key)
+        if include_self:
+            match_key = ''.join(match.conv.key)
+            if match_key not in match.conv.values:
+                match.conv.values.append(match_key)
         for value in match.conv.values:
             result = parts[:index] + [value] + parts[match.end:]
             queue.append((result, matched + 1, index + 1))
@@ -524,8 +576,10 @@ class StsDict(OrderedDict):
             if match is not None:
                 if match.end - index == 1:
                     has_atomic_match = True
-                if include_self and match.conv.key not in match.conv.values:
-                    match.conv.values.append(match.conv.key)
+                if include_self:
+                    match_key = ''.join(match.conv.key)
+                    if match_key not in match.conv.values:
+                        match.conv.values.append(match_key)
                 for value in match.conv.values:
                     result = parts[:index] + [value] + parts[match.end:]
                     queue.append((result, matched + 1, index + 1))
@@ -561,13 +615,13 @@ class Table(StsDict):
     def key_maxlen(self):
         """Get the maximal length of the keys.
         """
-        return max(len(Unicode.split(key)) for key in self)
+        return max(len(Unicode.split(key)) for key in self._dict)
 
     @lazyprop
     def key_headchars(self):
         """Get a set of the first char of the keys.
         """
-        return set(key[0] for key in self)
+        return set(key[0] for key in self._dict)
 
     def match(self, parts, pos, maxlen=math.inf):
         """Match a unicode composite at pos.
@@ -578,17 +632,19 @@ class Table(StsDict):
         Returns:
             an StsDictMatch or None if no match.
         """
-        parts = (Unicode.split(parts) if isinstance(parts, str) else
-                parts if isinstance(parts, list) else
-                list(parts))
+        parts = self._split(parts)
         if parts[pos][0] in self.key_headchars:
             i = self.key_maxlen
             i = min(i, min(len(parts), maxlen) - pos)
             while i >= 1:
                 end = pos + i
-                current = "".join(parts[pos:end])
-                if current in self:
-                    conv = StsDictConv(current, self[current])
+                current_parts = parts[pos:end]
+                current = "".join(current_parts)
+                try:
+                    conv = StsDictConv(current_parts, self._dict[current])
+                except KeyError:
+                    pass
+                else:
                     return StsDictMatch(conv, pos, end)
                 i -= 1
         return None
@@ -601,7 +657,71 @@ class Trie(StsDict):
 
     NOTE: The internal data format is different from base StsDict.
     """
-    def iter(self):
+    def __getitem__(self, key):
+        """Implementation of self[key].
+        """
+        trie = self._dict
+        try:
+            for k in self._split(key):
+                trie = trie[k]
+            return trie[""]
+        except KeyError:
+            raise KeyError(key)
+
+    def __contains__(self, item):
+        """Implementation of "item in self".
+        """
+        try:
+            self[item]
+        except KeyError:
+            return False
+        else:
+            return True
+
+    def __len__(self):
+        """Implementation of len(self).
+        """
+        return sum(1 for _ in self)
+
+    def __iter__(self):
+        """Implementation of iter(self).
+        """
+        yield from self.keys()
+
+    def keys(self):
+        """Get a generator of keys.
+        """
+        def recurse(trie):
+            for key in trie:
+                if key == "":
+                    yield ''.join(keystack)
+                else:
+                    keystack.append(key)
+                    triestack.append(trie[key])
+                    yield from recurse(triestack[-1])
+            keystack.pop()
+            triestack.pop()
+
+        keystack = [""]
+        triestack = [self._dict]
+        yield from recurse(triestack[-1])
+
+    def values(self):
+        """Get a generator of values.
+        """
+        def recurse(trie):
+            for key in trie:
+                if key == "":
+                    yield trie[key]
+                else:
+                    triestack.append(trie[key])
+                    yield from recurse(triestack[-1])
+            triestack.pop()
+
+        triestack = [self._dict]
+        yield from recurse(triestack[-1])
+
+    def items(self):
         """Get a generator of key-values pairs.
         """
         def recurse(trie):
@@ -616,7 +736,7 @@ class Trie(StsDict):
             triestack.pop()
 
         keystack = [""]
-        triestack = [self]
+        triestack = [self._dict]
         yield from recurse(triestack[-1])
 
     def add(self, key, values, skip_check=False):
@@ -628,15 +748,12 @@ class Trie(StsDict):
         """
         values = [values] if isinstance(values, str) else values
 
-        current = self
+        current = self._dict
         for i, composite in enumerate(Unicode.split(key)):
-            current = current.setdefault(composite, OrderedDict())
+            current = current.setdefault(composite, {})
 
         list_ = current.setdefault('', [])
-        if skip_check:
-            list_ += values
-        else:
-            list_ += [x for x in values if x not in list_]
+        list_ += values if skip_check else [x for x in values if x not in list_]
         return self
 
     def match(self, parts, pos, maxlen=math.inf):
@@ -648,25 +765,26 @@ class Trie(StsDict):
         Returns:
             an StsDictMatch or None if no match.
         """
-        parts = (Unicode.split(parts) if isinstance(parts, str) else
-                parts if isinstance(parts, list) else
-                list(parts))
-        trie = self
+        parts = self._split(parts)
+        trie = self._dict
         i = pos
         total = min(len(parts), maxlen)
         match = None
         match_end = None
         while i < total:
-            key = parts[i]
-            if key not in trie:
+            try:
+                trie = trie[parts[i]]
+            except KeyError:
                 break
-            trie = trie[key]
-            if "" in trie:
+            try:
                 match = trie[""]
+            except KeyError:
+                pass
+            else:
                 match_end = i + 1
             i = i + 1
         if match:
-            conv = StsDictConv("".join(parts[pos:match_end]), match)
+            conv = StsDictConv(parts[pos:match_end], match)
             return StsDictMatch(conv, pos, match_end)
         return None
 
@@ -692,66 +810,8 @@ class StsListMaker():
             }, ...]
         }
         """
-        def get_config_file(config, base_dir=None):
-            """Calculate the path of a config file.
-
-            1. Use it if it's an absolute path.
-            2. Assume relative to base_dir or CWD.
-            3. Assume relative to default config directory. (.json omissible)
-            4. If not found, assume relative to CWD.
-            """
-            if os.path.isabs(config):
-                return config
-
-            relative_config = os.path.join(base_dir, config) if base_dir is not None else config
-            if os.path.isfile(relative_config):
-                return relative_config
-
-            search_file = os.path.join(self.DEFAULT_CONFIG_DIR, config)
-            if os.path.isfile(search_file):
-                return search_file
-            if not config.lower().endswith('.json'):
-                for file in glob.iglob(glob.escape(search_file) + '.[jJ][sS][oO][nN]'):
-                    return file
-
-            return relative_config
-
-        def get_stsdict_file(stsdict):
-            """Calculate the path of a dictionary file.
-
-            1. Use it if it's an absolute path.
-            2. Assume relative to the config file.
-            3. Assume relative to default dictionary directory.
-            4. If not found, assume relative to the config file.
-            """
-            if os.path.isabs(stsdict):
-                return stsdict
-
-            search_file = os.path.join(config_dir, stsdict)
-            if os.path.isfile(search_file):
-                return search_file
-
-            search_file2 = os.path.join(self.DEFAULT_DICTIONARY_DIR, stsdict)
-            if os.path.isfile(search_file2):
-                return search_file2
-
-            return search_file
-
-        def check_update(output, filegroups):
-            if not os.path.isfile(output):
-                return True
-
-            for files in filegroups:
-                if isinstance(files, str):
-                    files = [files]
-                for file in files:
-                    if os.path.getmtime(file) > os.path.getmtime(output):
-                        return True
-
-            return False
-
         # locate and load the config file
-        config_file = get_config_file(config_name, base_dir)
+        config_file = self.get_config_file(config_name, base_dir=base_dir)
         config_dir = os.path.abspath(os.path.dirname(config_file))
 
         with open(config_file, "r", encoding="UTF-8") as f:
@@ -768,11 +828,11 @@ class StsListMaker():
             dest = os.path.join(output_dir or config_dir, dict_['file'])
             format = dict_['format']
             mode = dict_['mode']
-            files = [get_stsdict_file(f) if isinstance(f, str)
-                else [get_stsdict_file(i) for i in f]
+            files = [self.get_stsdict_file(f, base_dir=config_dir) if isinstance(f, str)
+                else [self.get_stsdict_file(i, base_dir=config_dir) for i in f]
                 for f in dict_['src']]
 
-            if not check_update(dest, files):
+            if not self.check_update(dest, files):
                 if not quiet: print(f'skip making (up-to-date): {dest}')
                 continue
 
@@ -794,12 +854,75 @@ class StsListMaker():
             elif format == 'jlist':
                 table.dumpjson(dest)
             elif format == 'tlist':
-                Trie().add_dict(table).dumpjson(dest)
+                Trie(table).dumpjson(dest)
             else:
                 raise ValueError(f'Specified format "{format}" is not supported.')
 
         return dest
 
+    def get_config_file(self, config, base_dir=None):
+        """Calculate the path of a config file.
+
+        1. Use it if it's an absolute path.
+        2. Assume relative to base_dir or CWD.
+        3. Assume relative to default config directory. (basename only; .json omissible)
+        4. If not found, return 2.
+        """
+        if os.path.isabs(config):
+            return config
+
+        relative_config = os.path.join(base_dir, config) if base_dir is not None else config
+        if os.path.isfile(relative_config):
+            return relative_config
+
+        if os.path.basename(config) == config:
+            search_file = os.path.join(self.DEFAULT_CONFIG_DIR, config)
+            if os.path.isfile(search_file):
+                return search_file
+            if not config.lower().endswith('.json'):
+                search_file = os.path.join(self.DEFAULT_CONFIG_DIR, config + '.json')
+                if os.path.isfile(search_file):
+                    return search_file
+
+        return relative_config
+
+    def get_stsdict_file(self, stsdict, base_dir=None):
+        """Calculate the path of a dictionary file.
+
+        1. Use it if it's an absolute path.
+        2. Assume relative to base_dir or CWD.
+        3. Assume relative to default dictionary directory. (basename only)
+        4. If not found, return 2.
+        """
+        if os.path.isabs(stsdict):
+            return stsdict
+
+        relative_stsdict = os.path.join(base_dir, stsdict) if base_dir is not None else stsdict
+        if os.path.isfile(relative_stsdict):
+            return relative_stsdict
+
+        if os.path.basename(stsdict) == stsdict:
+            search_file = os.path.join(self.DEFAULT_DICTIONARY_DIR, stsdict)
+            if os.path.isfile(search_file):
+                return search_file
+
+        return relative_stsdict
+
+    def check_update(self, output, filegroups):
+        """Check if the output file needs update.
+        """
+        if not os.path.isfile(output):
+            return True
+
+        for files in filegroups:
+            if isinstance(files, str):
+                files = [files]
+            for file in files:
+                if os.path.getmtime(file) > os.path.getmtime(output):
+                    return True
+
+        return False
+
     DEFAULT_CONFIG_DIR = os.path.join(os.path.dirname(__file__), 'data', 'config')
     DEFAULT_DICTIONARY_DIR = os.path.join(os.path.dirname(__file__), 'data', 'dictionary')
 
@@ -866,27 +989,31 @@ class StsConverter():
                 if isinstance(part, str):
                     yield part
                 else:
-                    old, news = part
-
-                    if self.options['mark']:
-                        if len(news) == 1 and old == news[0]:
-                            part = "{{" + old + "}}"
-                        else:
-                            part = "{{" + old + "->" + "|".join(news) + "}}"
-                    else:
-                        part = news[0]
+                    yield part[1][0]
 
+        def parts_to_marked_text(parts):
+            for part in parts:
+                if isinstance(part, str):
                     yield part
+                else:
+                    olds, news = part
+                    old = ''.join(olds)
+
+                    if len(news) == 1 and old == news[0]:
+                        yield "{{" + old + "}}"
+                    else:
+                        yield "{{" + old + "->" + "|".join(news) + "}}"
 
         def parts_to_html(parts):
             for part in parts:
                 if isinstance(part, str):
                     yield html.escape(part)
                 else:
-                    old, news = part
-                    content = f'<del hidden>{html.escape(old)}</del>'
+                    olds, news = part
+                    old = ''.join(olds)
+                    content = f'<del>{html.escape(old)}</del>'
                     for i, v in enumerate(news):
-                        content += f'<ins{" hidden" if i else ""}>{html.escape(v)}</ins>'
+                        content += f'<ins>{html.escape(v)}</ins>'
 
                     # classes
                     classes = ['sts-conv']
@@ -896,8 +1023,10 @@ class StsConverter():
                         classes.append('plural')
                     if old == news[0]:
                         classes.append('exact')
+                    if len(olds) == 1:
+                        classes.append('atomic')
 
-                    part = f'''<span tabindex="0" class="{' '.join(classes)}">{content}</span>'''
+                    part = f'''<span class="{' '.join(classes)}">{content}</span>'''
                     yield part
 
         conversion = self.convert(text)
@@ -909,11 +1038,14 @@ class StsConverter():
 <meta charset="UTF-8">
 <style>
 pre { white-space: pre-wrap; }
-span { font-weight: normal; border: thin dotted #AAA; }
-span.plural { background: #FFFF99; }
-span.exact { background: #DDDDFF; }
-span.single { background: #CCFFCC; }
-ins, del { text-decoration: none; }
+.sts-conv { font-weight: normal; border: thin dotted #AAA; }
+.sts-conv.plural { background: #FFFF99; }
+.sts-conv.exact { background: #DDDDFF; }
+.sts-conv.single { background: #CCFFCC; }
+.sts-conv ins, .sts-conv del { text-decoration: none; }
+.sts-conv del { display: none; }
+.sts-conv ins { display: none; }
+.sts-conv ins:first-of-type { display: inline; }
 </style>
 </head>
 <body>
@@ -924,29 +1056,30 @@ ins, del { text-decoration: none; }
             return ''.join(parts_to_html(conversion))
         elif self.options['format'] == 'json':
             return json.dumps(list(conversion), ensure_ascii=False)
+        elif self.options['format'] == 'txtm':
+            return ''.join(parts_to_marked_text(conversion))
         else:  # default: format = 'txt'
             return ''.join(parts_to_text(conversion))
 
-    def convert_file(self, input=None, output=None):
+    def convert_file(self, input=None, output=None, input_encoding='UTF-8', output_encoding='UTF-8'):
         """Convert input and write to output.
 
         Args:
             input: a file path or None for stdin.
             output: a file path or None for stdout.
         """
-        f = open(input, "r", encoding="UTF-8", newline="") if input else sys.stdin
+        f = open(input, "r", encoding=input_encoding, newline="") if input else sys.stdin
         text = f.read()
         f.close()
 
         conversion = self.convert_text(text)
 
-        f = open(output, "w", encoding="UTF-8", newline="") if output else sys.stdout
+        f = open(output, "w", encoding=output_encoding, newline="") if output else sys.stdout
         f.write(conversion)
-        f.close()
+        if f is not sys.stdout: f.close()
 
     default_options={
         'format': 'txt',
-        'mark': False,
         'exclude': None,
         }
 
@@ -954,18 +1087,22 @@ def main():
     def sort(args):
         """Sort a conversion list.
         """
-        input = args['input']
-        output = args['output'] or args['input']
+        inputs = args['file']
+        outputs = args['output']
 
-        Table().load(input).dump(output, sort=True)
+        for i, input in enumerate(inputs):
+            output = input if i >= len(outputs) else outputs[i]
+            Table().load(input).dump(output, sort=True)
 
     def swap(args):
         """Swap the key and value of a conversion list.
         """
-        input = args['input']
-        output = args['output'] or args['input']
+        inputs = args['file']
+        outputs = args['output']
 
-        Table().load(input).swap().dump(output, sort=True)
+        for i, input in enumerate(inputs):
+            output = input if i >= len(outputs) else outputs[i]
+            Table().load(input).swap().dump(output, sort=True)
 
     def merge(args):
         """Merge conversion lists.
@@ -985,7 +1122,7 @@ def main():
             print(f'{key} => {" ".join(values)}')
 
     def make(args):
-        """Compile conversion dictionary(s).
+        """Generate conversion dictionary(s).
         """
         configs = args['config']
         dir = args['dir']
@@ -997,59 +1134,72 @@ def main():
     def convert(args):
         """Convert a file using the given config.
         """
+        inputs = args['file']
+        outputs = args['output']
+        force_stdout = args['stdout']
         config = args['config']
-        input = args['input']
-        output = args['output']
         options={
             'format': args['format'],
-            'mark': args['mark'],
             'exclude': args['exclude'],
             }
+        input_encoding = args['in_enc']
+        output_encoding = args['out_enc']
 
         stsdict = StsListMaker().make(config, quiet=True)
         converter = StsConverter(stsdict, options)
-        converter.convert_file(input, output)
+
+        # read STDIN if no input file is specified
+        if not len(inputs):
+            inputs.append(None)
+
+        for i, input in enumerate(inputs):
+            output = None if force_stdout else input if i >= len(outputs) else outputs[i]
+            converter.convert_file(input, output, input_encoding, output_encoding)
 
     # define the parsers
-    parser = argparse.ArgumentParser(description=__doc__)
-    parser.add_argument('--version', default=False, action='store_true',
+    parser = argparse.ArgumentParser(prog='sts', description=__doc__)
+    parser.add_argument('--version', action='version', version=f'%(prog)s {__version__}',
         help="""show version information and exit""")
     subparsers = parser.add_subparsers(dest='func', metavar='COMMAND')
 
     # subcommand: convert
     parser_convert = subparsers.add_parser('convert',
         help=convert.__doc__, description=convert.__doc__)
-    parser_convert.add_argument('--config', '-c', default='s2t',
-        help="""the conversion config to use (default: s2t)""")
-    parser_convert.add_argument('--input', '-i', default=None,
-        help="""file to convert (default: stdin)""")
-    parser_convert.add_argument('--output', '-o', default=None,
-        help="""file to save the output (default: stdout)""")
-    parser_convert.add_argument('--format', '-f', default="txt",
-        choices=['txt', 'html', 'htmlpage', 'json'], metavar='FORMAT',
-        help="""output format (txt|html|htmlpage|json) (default: txt)""")
-    parser_convert.add_argument('--mark', '-m', default=False, action='store_true',
-        help="""mark converted chars for txt format""")
+    parser_convert.add_argument('file', nargs='*',
+        help="""file(s) to convert (default: STDIN)""")
+    parser_convert.add_argument('-c', '--config', default='s2t',
+        help="""the config to use, either a built-in config name or a path to a custom JSON file
+(built-in configs: s2t|t2s|s2tw|tw2s|s2twp|tw2sp|s2hk|hk2s|t2tw|tw2t|t2twp|tw2tp|t2hk|hk2t|t2jp|jp2t)
+(default: %(default)s)""")
+    parser_convert.add_argument('-f', '--format', default="txt",
+        choices=['txt', 'txtm', 'html', 'htmlpage', 'json'], metavar='FORMAT',
+        help="""output format (txt|txtm|html|htmlpage|json) (default: %(default)s)""")
     parser_convert.add_argument('--exclude',
-        help="""exclude text matching given regex from conversion, """
-            """optionally also replace with its "return" subgroup"""
-            )
+        help="""exclude text matching given regex from conversion, and replace it with the "return" subgroup value if exists""")
+    parser_convert.add_argument('--in-enc', default='UTF-8', metavar='ENCODING',
+        help="""encoding for input (default: %(default)s)""")
+    parser_convert.add_argument('--out-enc', default='UTF-8', metavar='ENCODING',
+        help="""encoding for output (default: %(default)s)""")
+    parser_convert.add_argument('-o', '--output', default=[], action='append',
+        help="""path to output (for the corresponding input) (default: to input)""")
+    parser_convert.add_argument('--stdout', default=False, action='store_true',
+        help="""write all converted text to STDOUT instead""")
 
     # subcommand: sort
     parser_sort = subparsers.add_parser('sort',
         help=sort.__doc__, description=sort.__doc__)
-    parser_sort.add_argument('input', nargs='?',
-        help="""file to sort""")
-    parser_sort.add_argument('output', nargs='?',
-        help="""file to save as (default: input)""")
+    parser_sort.add_argument('file', nargs='+',
+        help="""file(s) to sort""")
+    parser_sort.add_argument('-o', '--output', default=[], action='append',
+        help="""path to output (for the corresponding input) (default: to input)""")
 
     # subcommand: swap
     parser_swap = subparsers.add_parser('swap',
         help=swap.__doc__, description=swap.__doc__)
-    parser_swap.add_argument('input', nargs='?',
-        help="""file to swap""")
-    parser_swap.add_argument('output', nargs='?',
-        help="""file to save as (default: input)""")
+    parser_swap.add_argument('file', nargs='+',
+        help="""file(s) to swap""")
+    parser_swap.add_argument('-o', '--output', default=[], action='append',
+        help="""path to output (for the corresponding input) (default: to input)""")
 
     # subcommand: merge
     parser_merge = subparsers.add_parser('merge',
@@ -1070,19 +1220,17 @@ def main():
     # subcommand: make
     parser_make = subparsers.add_parser('make',
         help=make.__doc__, description=make.__doc__)
-    parser_make.add_argument('config', nargs='*',
-        help="""the config file(s) to compile (default: built-in compile)""")
-    parser_make.add_argument('--dir', '-d', default=None,
-        help="""the directory to save the output (default: config directory)""")
-    parser_make.add_argument('--quiet', '-q', default=False, action='store_true',
+    parser_make.add_argument('config', nargs='+',
+        help="""the config(s) to generate""")
+    parser_make.add_argument('-d', '--dir', default=None,
+        help="""the directory to save the output (default: relative to config)""")
+    parser_make.add_argument('-q', '--quiet', default=False, action='store_true',
         help="""do not show process information""")
 
     # parse the command
     args = vars(parser.parse_args())
     if args['func']:
         locals()[args['func']](args)
-    elif args['version']:
-        print(f'sts {__version__}')
     else:
         parser.parse_args(['-h'])
 
